function generate_matlabdata(threads, tests)
% matlab_performance
% This function tests the performance of a MATLAB implementation of Nolan's
% equations [1] realized by Mark Veillete and available for download at [2]
%    Inputs.
%           threads:   Max number of threads used by MATLAB. Can be a scalar
%                      or vector. Leave empty to set it automatically to
%                      the number of cores in the system.
%           tests:     Number of repetitions of each execution. Can be a
%                      scalar or vector. 10 by default.
%    Output: .mat file in generated_data folder.
%
% [1] Nolan, J. P. Numerical Calculation of Stable Densities and
%     Distribution Functions Stochastic Models, 1997, 13, 759-774
% [2] Mark S. Veillete. Alpha-Stable Distributions in MATLAB
%     http://math.bu.edu/people/mveillet/research.html
%
% Copyright (C) 2013. Javier Royuela del Val
%                     Federico Simmross Wattenberg

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; version 3 of the License.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; If not, see <http://www.gnu.org/licenses/>.
%
%
%  Javier Royuela del Val.
%  E.T.S.I. Telecomunicación
%  Universidad de Valladolid
%  Paseo de Belén 15, 47002 Valladolid, Spain.
%  jroyval@lpi.tel.uva.es    
%

% clear

addpath './matlab_veillete'
mark = '-\|/';

% Set alpha and beta
alpha = 0.75;
beta = 0.5;

% Standard distributions:
sigma = 1.0;
mu_0  = 0;

flagThreads = true;
if ~exist('threads','var') || isempty(threads)
    maxNumCompThreads('automatic');
    % Set again to get the number of treads
    threads = maxNumCompThreads('automatic');
    flagThreads = false;
end
   
if ~exist('tests', 'var')
    tests   = 100;
end

% 10000 points in aprox. the 5%-95% inter-quantile rank
% Values will change with alpha and beta.
x = (0:9999)*0.03125-40;

func = {'pdf','cdf'};
funname = {'stblpdf','stblcdf'};

reltol = [2e-14 1e-12 1e-10 1e-8 1e-6];
Ntol = length(reltol);

for Ntest = tests
    t_matlab=zeros(Ntol,Ntest,2);
    for ith = 1:length(threads)
        
        if flagThreads
            maxNumCompThreads(threads(ith));
            maxNumCompThreads(threads(ith));
        end
        
        for ifun = 1:2
            fprintf('\nGenerating %s performance data with %d threads\n', ...
                       func{ifun}, threads(ith));
            fprintf('0 -------------------- 100\n  ');
            for itest = 1:Ntest
                fprintf('%c',mark(mod(itest-1,4)+1));
                for itol = 1:Ntol
                    tic;
                    p = feval(funname{ifun},x,alpha,beta,sigma,mu_0,reltol(itol));
                    t_matlab(itol,itest,ifun) = toc;
                end
                fprintf('\b');
                % printf mark each 5% progress
                for j=floor(itest/Ntest*20):floor((itest+1)/Ntest*20)-1
                    fprintf('=');
                end
            end
            fprintf('\n');
        end

        t_matlab_pdf = t_matlab(:,:,1);
        t_matlab_cdf = t_matlab(:,:,2);

        filename = sprintf('./generated_data/matlab_performance_%dth_%dtest.mat', 0*threads(ith), Ntest);
        save(filename, 'alpha', 'beta', 'x', 'reltol', ...
             't_matlab_pdf', 't_matlab_cdf', 'Ntest');
    end
end

maxNumCompThreads('automatic');
end