/** @file stable_performance.c
 * @brief Libstable performance benchmark.
 *
 * This benchmark program is employed to evaluate the performance of Libstable
 * when calculating the PDF or CDF of standard alpha-stable distributions.
 * It obtains several calculations of the desired function at:
 *     - Selected values of alpha and beta.
 *     - A log-scaled sweep on abscissae axis.
 *     - A log-scaled sweep on precisions.
 *     - A selected number of threads of execution.
 *     - Each computation is repeated a predefined number of times to obtain
 * confidence intervals of libstable performance.
 *
 * Copyright (C) 2013. Javier Royuela del Val,
 *                     Federico Simmross Wattenberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Javier Royuela del Val.
 *  E.T.S.I. Telecomunicación,
 *  Universidad de Valladolid,
 *  Paseo de Belén 15, 47002 Valladolid, Spain.
 *  jroyval@lpi.tel.uva.es
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "stable.h"

#include <sys/time.h>
#include <time.h>

#define P 0 /** Use 0 parametrization */

/***********************
 * Main function
 *********************** */
int
main ( int argc, char * argv[] )
{
  int nfun = 0;         /* Funtion to evaluate: 1=>PDF, 2=> CDF */
  char *funname[] = {"", "pdf", "cdf"};
  int npointsacum = 0;  /* Total number of evaluations. */
  int kt, kp, i;     /* Aux indices */

  int Ntest = 100;	 /* Number of repetitions */
  int threads = 0;      /* Number of threads */

  double alpha = 0.75;  /* Alpha and beta values */
  double beta  = 0.5;
  double sigma = 1.0;   /* Standard distributions */
  double mu_0 = 0.0;

  /* Relative error tolerances */
  int    Ntol = 5;
  double reltol[] = {1.2e-14, 1e-12, 1e-10, 1e-8, 1e-6};
  double abstol = 1e-50; /* Fixed absolute error tolerance */

  /* Execution times */
  double t_total = 0, txdf;
  struct timeval t_ini, t_fin, tp_1, tp_2;
  time_t rawtime;
  struct tm * timeinfo;
  char hour[20];

  /* Data arrays */
  double * xdf, * err, * x;

  /* Output file and filename */
  FILE * ftimes;
  char nametimes[256];

  char marker[]= {'-','\\','|','/'};

  /* Stable distribution and pointer to PDF or CDF function */
  StableDist *dist;
  void ( *func ) ( StableDist *, const double *, const unsigned int,double *, double * );

  /* x-axis sweep:
   * Avoid too negative values where PDF and CDF approache zero.
   * -10 to   10 step 0.015625, (1280 points)
   *  10 to  100 step 0.12500,  ( 720 points)
   * 100 to 1000 step 1.00000,  ( 901 points) => total of 2901 points */
  /*
  int    Ndiv    = 3;
  int    Nxdiv[] = {     641,   721, 901  };
  double    x0[] = {     -10,    10, 100  };
  double xstep[] = { .015625,  0.25, 1.0  };
  */

  /* x-axis sweep
   * x values have been chosen to cover the 1%-99% inter-percentile range.
   * To achieve this, x0 and xstep must be adapted for each alpha and beta.
   */
  int       Nx = 10000;
  double  x0[] = { -40, 272.46875 };
  double xstep = .03125;
  
  /* Check input arguments */
  if ( argc == 4 )
    {
      nfun       = atoi ( argv[1] );
      threads = atoi ( argv[2] );
      Ntest   = atoi ( argv[3] );
    }
  if ( nfun < 1 || nfun > 2 || argc !=4 )
    {
      printf ( "Usage: stable_performance FUNC THREADS NTESTS\n" );
      printf ( "       FUNC = 1: PDF\n" );
      printf ( "       FUNC = 2: CDF\n" );
      exit ( 2 );
    }



  /* x-points aray */
  x   = ( double* ) malloc ( Nx*sizeof ( double ) );
  for ( i = 0; i < Nx; i++ )
    {
      x[i] = -x0[0] + xstep * i;
    }
  
  /* Memory allocation for xdf evaluation and error */
  xdf = ( double* ) malloc ( Nx*sizeof ( double ) );
  err = ( double* ) malloc ( Nx*sizeof ( double ) );

  /* StableDist creation */
  if ( ( dist = stable_create ( alpha,beta,sigma,mu_0,P ) ) == NULL )
    {
      printf ( "Error while creating stable distribution" );
      exit ( EXIT_FAILURE );
    }

  /* Set threads, relTOL and absTOL */
  stable_set_THREADS ( threads );
  stable_set_relTOL ( reltol[0] );
  stable_set_absTOL ( abstol );

  /* Set func pointer */
  if ( nfun == 2 ) func = &stable_cdf;
  else func = &stable_pdf;

  /* Open output file */
  sprintf ( nametimes, "libstable_performance_%s_%dth_%dtest.txt",
            funname[nfun],threads, Ntest );
  if ( ( ftimes = fopen ( nametimes, "wt" ) ) == NULL )
    {
      printf ( "Error while opening output file." );
      exit ( 3 );
    }
  
  /* Print header to file */
  fprintf ( ftimes,"NTOL %d\n", Ntol );
  fprintf ( ftimes,"NTESTS %d\n", Ntest );
  fprintf ( ftimes,"FUNCTION %d\n", nfun );
  fprintf ( ftimes,"ALPHA %1.2lf\n", alpha );
  fprintf ( ftimes,"BETA %1.2lf\n", beta );

  printf ( " - Libstable performance benchmark\n" );
  printf ( " - Alpha = %1.2lf, Beta = %1.2lf, Ntest  = %d\n\n", alpha, beta, Ntest );
  printf ( "   TIME      FUNC    RELTOL     0%% -------------------- 100%%\n" );

  /* Get start time*/
  gettimeofday ( &t_ini, NULL );
  /* relTol loop */
  for ( kp = 0; kp < Ntol; kp++ )
    {
      /* Print progress info */
      time ( &rawtime );
      timeinfo = localtime ( &rawtime );
      strftime ( hour, 20, "%X", timeinfo );
      printf ( " %s      %d   % 1.2e       ", hour, nfun, reltol[kp] );

      /* Print relTol header to file */
      fprintf ( ftimes,"RELTOL %1.2e\n", reltol[kp] );

      /* Set relTel to StableDist */
      stable_set_relTOL ( reltol[kp] );

      /* Execute function Ntest times */
      for ( kt = 0; kt < Ntest; kt++ )
        {
          putchar ( marker[kt%4] );
          fflush ( stdout );

          /* tic */
          gettimeofday ( &tp_1,NULL );
          /* Execute function */
          ( func ) ( dist, ( const double * ) x, Nx, xdf, err );
          /* toc */
          gettimeofday ( &tp_2,NULL );

          /* Calculate elapsed time in seconds */
          txdf = tp_2.tv_sec - tp_1.tv_sec + ( tp_2.tv_usec - tp_1.tv_usec ) / 1000000.0;

          /* Print to file */
          fprintf ( ftimes,"% 1.9lf ",txdf );

          /* Points evaluated till now */
          npointsacum += Nx;

          /* Print mark for each 5% progress */
          printf ( "\b" );
          for ( i = 0; i < floor ( ( kt + 1 ) * 20.0 / Ntest ) - floor ( kt * 20.0 / Ntest ); i++ )
            printf ( "=" );
        }

      fprintf ( ftimes,"\n" );
      printf ( "\n" );

    }
    
  fclose ( ftimes );
  ftimes = NULL;
  
  printf ( "\n" );

  gettimeofday ( &t_fin,NULL );
  t_total = t_fin.tv_sec-t_ini.tv_sec+ ( t_fin.tv_usec-t_ini.tv_usec ) /1000000.0;
  
  printf ( "    %d evaluations in %3.3lf seconds\n", npointsacum,t_total );
  printf ( "    Data in %s\n",nametimes );
  printf ( "    Avg. performance: %d points / %3.3lf s = %lf points/s\n\n",
           npointsacum,t_total, ( ( double ) npointsacum ) / ( ( double ) t_total ) );

  return 0;
}














