/** @file stable_precision.c
 * @brief Libstable precision benchmark.
 * 
 * This benchmark program is employed to obtain the data presented in Table 1
 * and Figure 3 of the manuscript.
 * It estimates the precision of Libstable when calculating the PDF or CDF of
 * standard alpha-stable distributions. It obtains several calculations of the
 * desired function at:
 *     - A finite set of points in the alpha-beta parameter space.
 *     - A log-scaled sweep on abscissae axis.
 *
 * Copyright (C) 2013. Javier Royuela del Val,
 *                     Federico Simmross Wattenberg.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Javier Royuela del Val.
 *  E.T.S.I. Telecomunicación,
 *  Universidad de Valladolid,
 *  Paseo de Belén 15, 47002 Valladolid, Spain.
 *  jroyval@lpi.tel.uva.es
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

#include "stable.h"

#define THREADS 0 /* Use as many threads as available */
#define P 0       /* Use 0 parametrization */

/* Compute the PDF or CDF of a standard alpha-stable distribution in a 
 * set of alpha and beta parameters and points x. Prints output to a txt file.
 */
int
compute (
  void ( * func ) ( StableDist*, const double*, const unsigned int, double*, double* ),
  char   * name,
  double reltol, double atol,
  double * alpha, int Na,
  double * beta,  int Nb,
  double * x,     int Nx )
{

  /* Standard distributions => sigma = 1, mu = 0 */
  double sigma = 1,  mu_0  = 0;

  /* Distribution structure */
  StableDist *dist = NULL;

  /* Execution times */
  struct timeval t_1, t_2;
  double t;

  /* Aux indices */
  int i, j, k;

  /* Output file */
  FILE * f;

  /* Data array */
  double * xdf;
  
  /* Memory allocation. */
  /* xdf will contain values for each beta and x */
  xdf = ( double* ) malloc ( Nx*Nb*sizeof ( double ) );

  /* Create output file */
  if ( ( f = fopen ( name, "wt" ) ) == NULL )
    {
      printf ( "Error while creating f file." );
      return -1;
    }

  /* Create distribution */
  if ( ( dist = stable_create ( alpha[0], beta[0], sigma, mu_0, P ) ) == NULL )
    {
      printf ( "Error while creating StableDistribution" );
      return -2;
    }

  /* Set distribution options */
  stable_set_THREADS ( THREADS );
  stable_set_relTOL ( reltol );
  stable_set_absTOL ( atol );

  /* Print header to console and file */
  printf ( "ALPHA# = %d BETA# = %d X# = %d\n", Na, Nb, Nx );

  fprintf ( f, "FUNCTION=%d ALPHAPOINTS=%d BETAPOINTS=%d XPOINTS=%d\n",
            1, Na, Nb, Nx );

  /* Init times */
  gettimeofday ( &t_1,NULL );

  /* Evaluation loop. */
  /* Iterates over alpha and beta */
  for ( j = 0; j < Na; j++ )
    {
      for ( i = 0; i < Nb; i++ )
        {
          /* Set alpha and beta */
          stable_setparams ( dist,alpha[j], beta[i], sigma, mu_0, P );

          /* Evaluate PDF or CDF function. */
          /* Concatenate output in xdf for each beta */
          ( func ) ( dist, ( const double * ) x, Nx, xdf + i * Nx, NULL );

          /* Print out progress */
          printf ( "        -> Alpha = %lf, Beta = %lf, %d points OK <-\n",
                   alpha[j], beta[i], Nx );
        } /* beta loop end */

      /* Write alpha division header in file*/
      fprintf ( f, "\nAlpha = %lf\n________x\\beta________\t", alpha[j] );
      for ( i = 0; i < Nb; i++ )
        {
          fprintf ( f, "%1.16e\t", beta[i] );
        }
      fprintf ( f, "\n" );

      /* first column of x values and function values for each x\beta */
      for ( k = 0; k < Nx; k++ )
        {
          /* x values */
          fprintf ( f, "%1.16e\t", x[k] );

          /* function values for that x for and each beta */
          for ( i = 0; i < Nb; i++ )
            {
              fprintf ( f, "%1.16e\t", * ( xdf + i*Nx + k ) );
            }

          fprintf ( f, "\n" );
        }

    } /* alpha loop end */

  /* Computation time */
  gettimeofday ( &t_2,NULL );
  t=t_2.tv_sec-t_1.tv_sec+ ( t_2.tv_usec-t_1.tv_usec ) /1000000.0;

  printf ( " -----> Done in %3.3lf seconds <-----\n\n",t );

  fclose ( f );

  return 0;
}

/***********************
 * Main function
 *********************** */
int
main ( int argc, char *argv[] )
{
  /* Set relative and absolute error tolerances to tiny values */
  double reltol  = 1.12e-14;
  double atol    = 1e-50;

  /* generate data for table_1 (n=1) or figure_3 (n=3) */
  int n = 0;

  if ( argc == 2 )
    {
      n = atoi ( argv[1] );
    }
  if (argc !=2 || (n != 1 && n != 3) )
    {
      printf ( "Usage: stable_precision FIG\n" );
      printf ( "       FIG = 1: Table 1 data\n" );
      printf ( "       FIG = 3: Figure 3 data\n" );
      exit ( 2 );
    }

  int Na, Nb;
  int Nx = 5417;
  double alpha[50];
  double beta[50];

  /* Output filename */
  char name[256];

  /* Aux indices */
  int i, j;

  /* Log-scaled x axis division*/
  double * x = ( double* ) malloc ( Nx*sizeof ( double ) );
  /* 7 segments */
  int    Ndiv = 7;
  /* Segments' edgees */
  double x0[]          = { -1000, -100,  -10,   -1,    1,   10,  100};
  /* Number of points per segment */
  double Nxdiv[]       = {   900,  720,  576, 1024,  576,  720,  901};
  /* Start position of each segment in the array */
  unsigned int Nx0[]   = {     0,  900, 1620, 2196, 3220, 3796, 4516};
  /* Points separation in each segment. Must be exactly representable */
  double xD[]          = { 1, .125, .015625, .001953125, .015625, .125, 1};

  /* Pointer to function (PDF or CDF)*/
  void ( *func ) ( StableDist *, const double*, const unsigned int, double*, double* );

  /* Initialize arrays */
  for ( i = 0; i < Ndiv; i++ )
    {
      for ( j = 0; j < Nxdiv[i]; j++ )
        {
          x[Nx0[i] + j] = x0[i] + xD[i] * j;
        }
    }
    
  switch ( n )
    {
    case 1: /* Table 1 data */
      Na = 5;
      Nb = 3;
      alpha[0] = 0.25;
      alpha[1] = 0.5;
      alpha[2] = 0.75;
      alpha[3] = 1;
      alpha[4] = 1.5;
      beta[0] = 0;
      beta[1] = 0.5;
      beta[2] = 1;

      break;

    case 3: /* Figure 3 data. Thinner sweep in alpha and beta */
      Na = 40;
      Nb = 17;
      for ( i = 0; i < Na; i++ )
        {
          alpha[i] = 0.05* ( i+1 );
        }
      for ( i = 0; i < Nb; i++ )
        {
          beta[i] = 0.0625*i;
        }

      break;

    default:
      printf ( "\nError: no valid n value (1 or 3)\n" );
      return -1;
    }




  /* PDF and CDF data computation */
  func = &stable_pdf;
  printf ( "PDF precision data computation:\n" );
  switch ( n )
    {
    case 1:
      sprintf ( name, "libstable_precision_pdf_table_1.txt" );
      break;
    case 3:
      sprintf ( name, "libstable_precision_pdf_figure_3.txt" );
      break;
    }

  compute ( func, name, reltol, atol, alpha, Na, beta, Nb, x, Nx );

  func = &stable_cdf;
  printf ( "CDF precision data computation:\n" );
  switch ( n )
    {
    case 1:
      sprintf ( name, "libstable_precision_cdf_table_1.txt" );
      break;
    case 3:
      sprintf ( name, "libstable_precision_cdf_figure_3.txt" );
      break;
    }
  compute ( func, name, reltol, atol, alpha, Na, beta, Nb, x, Nx );


  return 0;
}


