% function figure_3()
% figure_3:
% This function generates the figure 3 in the manuscript, with a summary of
% the relative error in the computation of the PDF and CDF.
%
% Copyright (C) 2013. Javier Royuela del Val
%                     Federico Simmross Wattenberg

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; version 3 of the License.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; If not, see <http://www.gnu.org/licenses/>.
%
%
%  Javier Royuela del Val.
%  E.T.S.I. Telecomunicación
%  Universidad de Valladolid
%  Paseo de Belén 15, 47002 Valladolid, Spain.
%  jroyval@lpi.tel.uva.es    
%

clear

addpath('aux/','aux/matlab_exportfig/','aux/export_fig');
set(0,'defaulttextinterpreter','latex');

datapath = '../generated_data/';

pdf_filename = [datapath 'libstable_precision_pdf_figure_3.txt'];
cdf_filename = [datapath 'libstable_precision_cdf_figure_3.txt'];
gt_pdf_filename = [datapath 'nolan_precision_pdf_figure_3.txt'];
gt_cdf_filename = [datapath 'nolan_precision_cdf_figure_3.txt'];
filenames={pdf_filename,cdf_filename,gt_pdf_filename,gt_cdf_filename};

checks = [exist(pdf_filename,'file')~=2; ...
          exist(cdf_filename,'file')~=2; ...
          exist(gt_pdf_filename,'file')~=2; ...
          exist(gt_cdf_filename,'file')~=2];
      
if (any(checks))
    fprintf(2,'Some requiered data is not present. Missing files:\n');
    for i = find(checks)
        fprintf(2, '  %s\n', filenames{i});
    end
    fprintf(1,'  Please make sure to generate required data with the provided scripts .\n');
    fprintf(1,'  To use precomputed data, uncompress it in ''generated_data'' folder.\n');
end


[alpha_s,  beta_s,  x_s,  pdf_s,  cdf_s]  = load_libstable_precision(pdf_filename, cdf_filename);
[alpha_gt, beta_gt, x_gt, pdf_gt, cdf_gt] = load_nolan_precision(gt_pdf_filename, gt_cdf_filename);

% NOTE: Ground truth fails for alpha<=0.05
% NOTE: Ground truth data shows wrong behaviour for values below ~ 1e-16
% as an example, look at the pdf when alpha=1, beta=1 and x goes negative
% Calculate absolute and relative errors.
% Where abserr == < tiny, relerr = 0 (avoid 0/0 = NaN)
tiny = 1e-20;
pdf_abserr = abs(pdf_s-pdf_gt);
pdf_relerr = pdf_abserr./pdf_gt;
pdf_relerr(pdf_abserr <= tiny) = 0;

cdf_abserr = abs(cdf_s-cdf_gt);
cdf_relerr = cdf_abserr./cdf_gt;
cdf_relerr(cdf_abserr <= tiny) = 0;

% Avoid extreme values to alter mean. As an example, a single point with
% relative error of 1e-10 with the rest of the errors set to zero raises
% the average relative error to 5.5e-14.
% Remove only 0.5% of higher values (peaks) and 0.5% of lower values (zeros)
pdf_avgerr = squeeze(trimmean(pdf_relerr,99));
cdf_avgerr = squeeze(trimmean(cdf_relerr,99));

% Median filt on log values (scale used for representation).
h = 3;
pdf_f = exp(medfilt2(log(pdf_avgerr),[h h],'symmetric'));
cdf_f = exp(medfilt2(log(cdf_avgerr),[h h],'symmetric'));

% Adjust to same color log-scale aprox between 0 and 1
pdf_c = (log10(pdf_f) + 16) ./ (-11 + 16);
cdf_c = (log10(cdf_f) + 16) ./ (-11 + 16);

% Surf plots

%%% PDF %%%
figure(31);
surf(alpha_s,beta_s,pdf_f,pdf_c);
set(gca,'zscale','log');
zlim([1e-16 1e-12])
shading('faceted');
caxis([0 1])
view(-13,14);
lightangle(-10,38);
set(findobj(gca,'type','surface'),...
    'FaceLighting','phong',...
    'AmbientStrength',1,'DiffuseStrength',.8,...
    'SpecularStrength',.5,'SpecularExponent',25,...
    'BackFaceLighting','unlit');
xlabel('$\alpha$');
ylabel('$\beta$');
zlabel('$\varepsilon_{r}$');
set(gca,'ZminorGrid','on')
title('PDF precision analysis');

%%% CDF %%%
figure(32);
surf(alpha_s,beta_s,cdf_f,cdf_c);
caxis([0 1]);
set(gca,'zscale','log');
zlim([1e-16 1e-12])
shading('faceted');
view(-13,14);
lightangle(-10,38);
set(findobj(gca,'type','surface'),...
    'FaceLighting','phong',...
    'AmbientStrength',1,'DiffuseStrength',.8,...
    'SpecularStrength',.5,'SpecularExponent',25,...
    'BackFaceLighting','unlit');
xlabel('$\alpha$');
ylabel('$\beta$');
zlabel('$\varepsilon_{r}$');
set(gca,'ZminorGrid','on')
title('CDF precision analysis');

%%
if false
asel = find(ismember(alpha_s,[0.95 1 1.05 1.7 1.8]));
bsel = find(ismember(beta_s,[0.25 0.5 1]));

figure(33);
for ia = 1:5
    for ib = 1:3
        subplot(5,3,ib+3*(ia-1));
        semilogy(x_s,pdf_gt(:,bsel(ib),asel(ia)),x_s,pdf_s(:,bsel(ib),asel(ia)),x_s,pdf_relerr(:,bsel(ib),asel(ia)));
        ylim([1e-20 10]);
        title(sprintf('$\\alpha = %0.2f, \\beta = %0.2f $',alpha_s(asel(ia)),beta_s(bsel(ib))));
        ylabel('$f_{X}(x;\alpha,\beta)$');
        xlabel('$x$');
    end
end

figure(34);
for ia = 1:5
    for ib = 1:3
        subplot(5,3,ib+3*(ia-1));
        semilogy(x_s,cdf_gt(:,bsel(ib),asel(ia)),x_s,cdf_s(:,bsel(ib),asel(ia)),x_s,cdf_relerr(:,bsel(ib),asel(ia)));
        ylim([1e-20 10]);
        title(sprintf('$\\alpha = %0.2f, \\beta = %0.2f $',alpha_s(asel(ia)),beta_s(bsel(ib))));
        ylabel('$F_{X}(x;\alpha,\beta)$');
        xlabel('$x$');
    end
end

end

    OPT=struct('Bounds','tight',...
               'Color' ,'rgb'  ,...
               'Width' ,'3.5'    ,...
               'Height','2.5' ,...
               'FontEncoding','latin1',...
               'FontMode'    ,'fixed',...
               'LockAxes'    ,'0',...
               'Renderer', 'painters');
           % 'FontSize'    ,'9',...
           
    exportfig(31,'../figure_3_pdf.eps',OPT);
    exportfig(32,'../figure_3_cdf.eps',OPT);
%    export_fig('../figure_3_pdf.eps','-transparent','-eps','-r2000', 31);
%    export_fig('../figure_3_cdf.eps','-transparent','-eps','-r2000', 32);
