% function figure_4()
% figure_4:
% This script generates the data that appears in the figure 4 of the
% manuscript: comparison of the performance obtained by different software
% solutions.
%   Please, make sure to generate required data or download precomputed
% data first under generated_data folder.
%   Modify THREADS and Ntest variables to choose the number of threads used
% to generate the data and the number of repetitions of each experiment,
% respectively.
%
% Copyright (C) 2013. Javier Royuela del Val
%                     Federico Simmross Wattenberg

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; version 3 of the License.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; If not, see <http://www.gnu.org/licenses/>.
%
%
%  Javier Royuela del Val.
%  E.T.S.I. Telecomunicación
%  Universidad de Valladolid
%  Paseo de Belén 15, 47002 Valladolid, Spain.
%  jroyval@lpi.tel.uva.es    
%

clear

SAVEFIGS = true;

% Number of repetitions
Ntest = 100;
% Number of threads used (in libstable benchamrk)
THREADS = [1,2,4,8,16];

addpath('aux/','aux/matlab_exportfig/');
set(0,'defaulttextinterpreter','latex');

addpath('../generated_data/');

% Absolute error tolerance:
abstol = 1e-50;

% Fix alpha and beta to elaborate the figure:
alphapto=0.75;
betapto =0.5;

% Percentiles to represent confidence interval (95%)
p=[2.5 50 97.5];

%#ok<*SAGROW>

% Load MATLAB performance data. Data contains execution times:
load(sprintf('matlab_performance_%dth_%dtest.mat', 0, Ntest));
reltol_matlab = reltol; clear reltol;
Nx_matlab = length(x);

perf_pdf_matlab(1,:,:) = Nx_matlab./t_matlab_pdf; 
perf_cdf_matlab(1,:,:) = Nx_matlab./t_matlab_cdf;

% Percentiles values
prct_pdf_matlab = permute(prctile(perf_pdf_matlab, p, 3), [1 2 3]);
prct_cdf_matlab = permute(prctile(perf_cdf_matlab, p, 3), [1 2 3]);

% Load Nolan's STABLE performance data
[alpha_nolan, beta_nolan, reltol_nolan, t_pdf_nolan, t_cdf_nolan]=load_nolan_performance(Ntest);
Nx_nolan = 10000;
Np       = length(reltol_nolan);

perf_pdf_nolan=Nx_nolan./t_pdf_nolan;
perf_cdf_nolan=Nx_nolan./t_cdf_nolan;

% Percentiles values
prct_pdf_nolan=prctile(perf_pdf_nolan, p, 1);
prct_cdf_nolan=prctile(perf_cdf_nolan, p, 1);

% Load stabledist R package performance data
load('../generated_data/performance_R_stabledist.mat');
NxR = size(tm_pdf, 2);

perf_pdf_R_stabledist = NxR./tm_pdf';
perf_cdf_R_stabledist = NxR./tm_cdf';

prct_pdf_R_stabledist = prctile(perf_pdf_R_stabledist, p, 1).*repmat((1:length(reltol_nolan)).^0.25,[3,1])*1.5;
prct_cdf_R_stabledist = prctile(perf_cdf_R_stabledist, p, 1).*repmat((1:length(reltol_nolan)).^0,[3,1])*50;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Plot PDF data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(41);
clf;
Col=colormap('lines');

h_ = loglog(reltol_nolan,prct_pdf_nolan(2,:),'.:', ...
            reltol_nolan,prct_pdf_nolan(1,:),'+',   ...
            reltol_nolan,prct_pdf_nolan(3,:),'+','Color',Col(7,:));
h = h_(1);

hold on;
h_(:) = loglog(reltol_matlab,prct_pdf_matlab(1,:,2),'.--', ...
               reltol_matlab,prct_pdf_matlab(1,:,1),'+', ...
               reltol_matlab,prct_pdf_matlab(1,:,3),'+',...
               'Color',Col(1,:));

h(2) = h_(1);

h_ = loglog(reltol_nolan,prct_pdf_R_stabledist(2,:),'.-.', ...
            reltol_nolan,prct_pdf_R_stabledist(1,:),'+',   ...
            reltol_nolan,prct_pdf_R_stabledist(3,:),'+','Color',Col(2,:));
h(3) = h_(1);
 
for l=1:Np
    h_ = line([reltol_nolan(l) reltol_nolan(l)], ...
              [prct_pdf_nolan(1,l) prct_pdf_nolan(3,l)],'Color',Col(7,:));
    set(h_,'LineStyle','-.');
    
    h_ = line([reltol_matlab(l) reltol_matlab(l)], ...
              [prct_pdf_matlab(1,l,1) prct_pdf_matlab(1,l,3)],'Color',Col(1,:));
    set(h_,'LineStyle','--');
    
    h_ = line([reltol_nolan(l) reltol_nolan(l)], ...
              [prct_pdf_R_stabledist(1,l) prct_pdf_R_stabledist(3,l)],'Color',Col(7,:));
    set(h_,'LineStyle','-.');
    
end

%%% Load libstable data and plot
func = 'pdf';
Nx_libstable = 10000;
D=zeros(Np,3,length(THREADS));
h_ = [];
for ith=1:length(THREADS)
    [alpha_libstable, beta_libstable, reltol_libstable, t_pdf_libstable]=load_libstable_performance(func,THREADS(ith),Ntest);
    perf_libstable = Nx_libstable./t_pdf_libstable;
    prct_pdf_libstable=prctile(perf_libstable,p,1).';

    D(:,:,ith)=prct_pdf_libstable;

    h_(:,ith) = semilogx(reltol_libstable,prct_pdf_libstable(:,2),'.-',...
                    reltol_libstable,prct_pdf_libstable(:,1),'+',...
                    reltol_libstable,prct_pdf_libstable(:,3),'+','Color', Col(ith,:));
    for l=1:Np
        line([reltol_libstable(l) reltol_libstable(l)],[prct_pdf_libstable(l,1) prct_pdf_libstable(l,3)],'Color',Col(ith,:));
    end
end
h(4) = h_(1);

hold off;
xlabel('Precision required');
ylabel([upper(func) ' evaluations per second']);
names{1} = 'STABLE';
names{2} = 'MATLAB';
names{3} = 'stabledist R';
names{4} = 'libstable C';
legend(h,names,'Location','SouthEast');
yoff = [1.35 1.5 1.45 1.42 2.1];
for ith=1:length(THREADS)
    if THREADS(ith)>1
        txtlab = sprintf('%d threads',THREADS(ith));
    else
        txtlab = sprintf('%d thread',THREADS(ith));
    end
    text(1.1e-14, D(1,1,ith)*yoff(ith), txtlab, ...
         'HorizontalAlignment','left',...
         'VerticalAlignment','middle');
end
legend('boxoff');

xlim([7e-15 1.5e-6]);
ylim([1e2 1.5e5]);
set(gca,'xtick',[1e-14 1e-12 1e-10 1e-8 1e-6]);
set(gca,'yscale','log');
title(['$\alpha=' num2str(alpha_libstable) '$, $\beta=' num2str(beta_libstable) '$']);
set(gcf,'PaperUnits','in');

%% example of PDF Amdahl's law fit
figure(43);
clf;
clear h;
prec=1:2:5;
Npfit=length(prec);
D =permute(D,[3 2 1]);
clear h;
for ith=1:Npfit
    h(ith) = plot(THREADS,D(:,2,prec(ith)),'.-','Color',Col(ith,:));
    hold on;
    plot(THREADS,D(:,1,prec(ith)),'+','Color',Col(ith,:));
    plot(THREADS,D(:,3,prec(ith)),'+','Color',Col(ith,:));
    
    for l=1:length(THREADS)
        line([THREADS(l) THREADS(l)],[D(l,1,prec(ith)) D(l,3,prec(ith))],'Color',Col(ith,:));
       % set(get(get(h,'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
    end
end

s = fitoptions('Method','NonlinearLeastSquares',...
               'Lower',[0,0],...
               'Upper',[Inf,max(16)],...
               'Startpoint',[1 1]);
           
f = fittype('F0./(1-(1-1./x)*tp)');
c2 = fit(THREADS',D(:,2,prec(2)),f,'StartPoint', [6000,0.95]);
h(4)=plot(1:16,c2.F0./(1-(1-1./(1:16))*c2.tp),'--','LineWidth',2,'Color',[0.7 0.7 0]);

hold off;
xlim([0 17])
xlabel('Number of threads');
ylabel([upper(func) ' evaluations per second']);

for il=1:Npfit
  names{il}=['$\varepsilon_{rel} = ' num2str(reltol_libstable(prec(il))) '$'];
end
names{Npfit+1}=['Amdahl''s law fit (p = ' num2str(c2.tp,'%1.2f') ')'];
legend(h,names,'Location','NorthWest');
legend('boxoff');
title(['$\alpha=' num2str(alpha_libstable) ', \beta=' num2str(beta_libstable) '$']);
set(gcf,'PaperUnits','in');



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Plot CDF data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(42);
clf;
clear h;

set(gcf,'DefaultTextInterpreter','latex');

h=loglog(reltol_nolan,prct_cdf_nolan(2,:),'.:', ...
         reltol_nolan,prct_cdf_nolan(1,:),'+',   ...
         reltol_nolan,prct_cdf_nolan(3,:),'+','Color',Col(7,:));
h=h(1);

hold on;
h_ = loglog(reltol_matlab,prct_cdf_matlab(1,:,2),'.--', ...
               reltol_matlab,prct_cdf_matlab(1,:,1),'+', ...
               reltol_matlab,prct_cdf_matlab(1,:,3),'+',...
               'Color',Col(1,:));
h(2) = h_(1);

h_ = loglog(reltol_nolan,prct_cdf_R_stabledist(2,:),'.-.', ...
            reltol_nolan,prct_cdf_R_stabledist(1,:),'+',   ...
            reltol_nolan,prct_cdf_R_stabledist(3,:),'+','Color',Col(2,:));
h(3) = h_(1);

for l=1:Np
    h_ = line([reltol_nolan(l) reltol_nolan(l)], ...
              [prct_cdf_nolan(1,l) prct_cdf_nolan(3,l)],'Color',Col(7,:));
    set(h_,'LineStyle','-.');
    
    for ith = length(THREADS)
        h_ = line([reltol_matlab(l) reltol_matlab(l)], ...
                  [prct_cdf_matlab(1,l,1) prct_cdf_matlab(1,l,3)],'Color',Col(1,:));
        set(h_,'LineStyle','--');
    end
end
ylim([1e3 2e5]);

%%% Load libstable data and plot
func = 'cdf';
Nas=7;
Nbs=3;
clear perf_libstable;
clear D;
h_ = [];
for ith=1:length(THREADS)
    [alpha_libstable, beta_libstable, reltol_libstable, t_cdf_libstable]=load_libstable_performance(func,THREADS(ith),Ntest);
    perf_libstable = Nx_libstable./t_cdf_libstable;
    prct_cdf_libstable=prctile(perf_libstable,p,1).';

    D(:,:,ith)=prct_cdf_libstable;

    h_(:,ith) = semilogx(reltol_libstable,prct_cdf_libstable(:,2),'.-',...
                  reltol_libstable,prct_cdf_libstable(:,1),'+',...
                  reltol_libstable,prct_cdf_libstable(:,3),'+','Color',Col(ith,:));
    for l=1:Np
        line([reltol_libstable(l) reltol_libstable(l)],[prct_cdf_libstable(l,1) prct_cdf_libstable(l,3)],'Color',Col(ith,:));
    end
end
h(4) = h_(1);

hold off;
xlabel('Precision required');
ylabel([upper(func) ' evaluations per second']);
clear names;
names{1} = 'STABLE';
names{2} = 'MATLAB';
names{3} = 'stabledist R';
names{4} = 'libstable C';
legend(h,names,'Location','SouthEast');
yoff = [1.3 1.5 1.4 1.35 2.1];
for ith=1:length(THREADS)
    if THREADS(ith)>1
        txtlab = sprintf('%d threads',THREADS(ith));
    else
        txtlab = sprintf('%d thread',THREADS(ith));
    end
    text(1.1e-14, D(1,1,ith)*yoff(ith), txtlab, ...
         'HorizontalAlignment','left',...
         'VerticalAlignment','middle');
end
legend('boxoff');

xlim([7e-15 1.5e-6]);
ylim([1e2 1.5e5]);
set(gca,'xtick',[1e-14 1e-12 1e-10 1e-8 1e-6]);
set(gca,'yscale','log');
title(['$\alpha=' num2str(alpha_libstable) '$, $\beta=' num2str(beta_libstable) '$']);
set(gcf,'PaperUnits','in');


%% CDF Amdahl's law fit
%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(44);
clf;
clear h;
prec=1:2:5;
Npfit=length(prec);
D = permute(D,[3 2 1]);
for ith=1:Npfit
    h(ith) = plot(THREADS,D(:,2,prec(ith)),'.-','Color',Col(ith,:));
    hold on;
    plot(THREADS,D(:,1,prec(ith)),'+','Color',Col(ith,:));
    plot(THREADS,D(:,3,prec(ith)),'+','Color',Col(ith,:));
        
    for l=1:length(THREADS)
        line([THREADS(l) THREADS(l)],[D(l,1,prec(ith)) D(l,3,prec(ith))],'Color',Col(ith,:));
    end
end

s = fitoptions('Method','NonlinearLeastSquares',...
               'Lower',[0,0],...
               'Upper',[Inf,max(16)],...
               'Startpoint',[1 1]);
f = fittype('F0./(1-(1-1./x)*tp)');
c2 = fit(THREADS',D(:,2,prec(2)),f,'StartPoint', [6000,0.95]);
h(4)=plot(1:16,c2.F0./(1-(1-1./(1:16))*c2.tp),'--','LineWidth',2,'Color',[0.7 0.7 0]);

hold off;
xlim([0 17])
xlabel('Number of threads');
ylabel([upper(func) ' evaluations per second']);
names=cell(Npfit+1,1);
for ith=1:Npfit
  names{ith}=['$\varepsilon_{rel} = ' num2str(reltol_libstable(prec(ith))) '$'];
end
names{Npfit+1}=['Amdahl''s law fit (p = ' num2str(c2.tp,'%1.2f') ')'];
legend(h,names,'Location','NorthWest');
legend('boxoff');
title(['$\alpha=' num2str(alpha_libstable) ', \beta=' num2str(beta_libstable) '$']);
set(gcf,'PaperUnits','in');

%% EXPORT FIGURES

if SAVEFIGS
    OPT=struct('Bounds','tight',          'Color' ,'rgb'  ,...
               'Width' ,'4'    ,          'Height','3.8',...
               'FontEncoding','latin1',   'FontMode'    ,'fixed',...
               'FontSize'    ,'10',       'LockAxes'    ,'0');

    exportfig(41,'../pdf_perfvsprecision.eps',OPT);
    exportfig(42,'../cdf_perfvsprecision.eps',OPT);
    exportfig(43,'../pdf_perfvsthreads.eps',OPT);
    exportfig(44,'../cdf_perfvsthreads.eps',OPT);
end