%function table_1()
% table_1:
% This function generates the data that appears in table 1 of the
% manuscript, with the relative error in the computation of the PDF and the
% CDF in three differenced intervals.
%
% Copyright (C) 2013. Javier Royuela del Val
%                     Federico Simmross Wattenberg

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; version 3 of the License.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; If not, see <http://www.gnu.org/licenses/>.
%
%
%  Javier Royuela del Val.
%  E.T.S.I. Telecomunicación
%  Universidad de Valladolid
%  Paseo de Belén 15, 47002 Valladolid, Spain.
%  jroyval@lpi.tel.uva.es    
%

clear

addpath('aux/','aux/matlab_exportfig/');
set(0,'defaulttextinterpreter','latex');

addpath('../generated_data');

pdf_filename = 'libstable_precision_pdf_table_1.txt';
cdf_filename = 'libstable_precision_cdf_table_1.txt';
gt_pdf_filename = 'nolan_precision_pdf_table_1.txt';
gt_cdf_filename = 'nolan_precision_cdf_table_1.txt';
filenames={pdf_filename,cdf_filename,gt_pdf_filename,gt_cdf_filename};

checks = [exist(pdf_filename,'file')~=2; ...
          exist(cdf_filename,'file')~=2; ...
          exist(gt_pdf_filename,'file')~=2; ...
          exist(gt_cdf_filename,'file')~=2];
      
if (any(checks))
    fprintf(2,'Some requiered data is not present. Missing files:\n');
    for i = find(checks)
        fprintf(2, '  ../generated_data/%s\n', filenames{i});
    end
    fprintf(1,'  Please make sure to generate required data with the provided scripts .\n');
    fprintf(1,'  To use precomputed data, uncompress it in ''generated_data'' folder.\n');
end


[alpha_s,  beta_s,  x_s,  pdf_s,  cdf_s]  = load_libstable_precision(pdf_filename, cdf_filename);
[alpha_gt, beta_gt, x_gt, pdf_gt, cdf_gt] = load_nolan_precision(gt_pdf_filename, gt_cdf_filename);

% NOTE: Ground truth data shows wrong behaviour for values below ~ 1e-16
% as an example, look at the pdf when alpha=1, beta=1 and x goes negative
% Calculate absolute and relative errors.
% Where abserr == < tiny, relerr = 0 (avoid 0/0 = NaN)
tiny = 1e-20;
pdf_abserr = abs(pdf_s-pdf_gt);
pdf_relerr = pdf_abserr./pdf_gt;
pdf_relerr(pdf_abserr <= tiny) = 0;

cdf_abserr = abs(cdf_s-cdf_gt);
cdf_relerr = cdf_abserr./cdf_gt;
cdf_relerr(cdf_abserr <= tiny) = 0;

% Group errors in 3 intervals and compute average.
% Avoid extreme values to alter mean. As an example, a single point with
% relative error of 1e-10 with the rest of the errors set to zero raises
% the average relative error to 5.5e-14.
% Remove only 0.5% of higher values (peaks) and 0.5% of lower values (zeros)
n=[1, 1620, 3796, 5417]; % [-1000,10) ; [-10,10) ; (10,1000] 
pdf_avgerr = zeros([3,length(beta_s),length(alpha_s)]);
cdf_avgerr = zeros([3,length(beta_s),length(alpha_s)]);
for i=1:3
     pdf_avgerr(i,:,:) = trimmean(pdf_relerr(n(i):n(i+1),:,:),99);
     cdf_avgerr(i,:,:) = trimmean(cdf_relerr(n(i):n(i+1),:,:),99);
end

figure(11);
for ia = 1:5
    for ib = 1:3
        subplot(5,3,ib+3*(ia-1));
        semilogy(x_s,pdf_gt(:,ib,ia),x_s,pdf_s(:,ib,ia),x_s,pdf_relerr(:,ib,ia));
        ylim([1e-20 10]);
        title(sprintf('$\\alpha = %0.2f, \\beta = %0.2f $',alpha_s(ia),beta_s(ib)));
        ylabel('$f_{X}(x;\alpha,\beta)$');
        xlabel('$x$');
    end
end
figure(12);
for ia = 1:5
    for ib = 1:3
        subplot(5,3,ib+3*(ia-1));
        semilogy(x_s,cdf_gt(:,ib,ia),x_s,cdf_s(:,ib,ia),x_s,cdf_relerr(:,ib,ia));
        ylim([1e-20 10]);
        title(sprintf('$\\alpha = %0.2f, \\beta = %0.2f $',alpha_s(ia),beta_s(ib)));
        ylabel('$F_{X}(x;\alpha,\beta)$');
        xlabel('$x$');
    end
end


fprintf(1,'\n  PDF results\n');
fprintf(1,'==================================================================\n');
fprintf(1,'                                  x-axis interval\n');
fprintf(1,'                        ------------------------------------------\n');
fprintf(1,'alpha\tbeta\t\t[-1000,-10)\t[-10,10)\t[10,1000]\n');
fprintf(1,'------------------------------------------------------------------\n');
for ia=1:length(alpha_s)
    for ib=1:length(beta_s)
        if ib==2, fprintf(1,'%0.2g',alpha_s(ia)); end
        fprintf(1,'\t %0.2g\t\t%0.2g\t\t%0.2g\t\t%0.2g\n',beta_s(ib),pdf_avgerr(1,ib,ia),pdf_avgerr(2,ib,ia),pdf_avgerr(3,ib,ia));
    end
    fprintf(1,'------------------------------------------------------------------\n');
end

fprintf(1,'\n  CDF results\n');
fprintf(1,'==================================================================\n');
fprintf(1,'                                  x-axis interval\n');
fprintf(1,'                        ------------------------------------------\n');
fprintf(1,'alpha\tbeta\t\t[-1000,-10)\t[-10,10)\t[10,1000]\n');
fprintf(1,'------------------------------------------------------------------\n');
for ia=1:length(alpha_s)
    for ib=1:length(beta_s)
        if ib==2, fprintf(1,'%0.2g',alpha_s(ia)); end
        fprintf(1,'\t %0.2g\t\t%0.2g\t\t%0.2g\t\t%0.2g\n',beta_s(ib),cdf_avgerr(1,ib,ia),cdf_avgerr(2,ib,ia),cdf_avgerr(3,ib,ia));
    end
    fprintf(1,'------------------------------------------------------------------\n');
end