/** @file nolan_performance.c
 * @brief John Nolan's STABLE performance benchmark.
 *
 * This benchmark program is employed to evaluate the performance of John
 * Nolan's STABLE software when calculating the PDF or CDF of standard
 * alpha-stable distributions. It obtains several calculations of the desired
 * function at:
 *   - Selected values of alpha and beta.
 *   - A log-scaled sweep on abscissae axis.
 *   - A log-scaled sweep on precisions.
 *   - A selected number of threads of execution.
 *   - Each computation is repeated a predefined number of times to obtain
 * confidence intervals of John Nolan's STABLE performance.
 *
 * Each computation is repeated a predefined number of times to obtain
 * confidence intervals of libstable performance.
 *
 * Copyright (C) 2013. Javier Royuela del Val,
 *                     Federico Simmross Wattenberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Javier Royuela del Val.
 *  E.T.S.I. Telecomunicación,
 *  Universidad de Valladolid,
 *  Paseo de Belén 15, 47002 Valladolid, Spain.
 *  jroyval@lpi.tel.uva.es
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef _WIN32
#include <windows.h>
#else
#include <sys/time.h>
#include <time.h>
#endif

//#define min(a,b) ((a<b) ? (a) : (b))
#define P 0 /* Use 0 parametrization */

int
main ( int argc, char *argv[] )
{
  int nfun = 0;         /* Funtion to evaluate: 1=>PDF, 2=> CDF */
  char *funname[] = {"", "pdf", "cdf"};

  int npointsacum = 0;  /* Total number of evaluations. */
  int kt, i, j;     /* Aux indices */

  int Ntest = 100;	 /* Number of repetitions */

  double alpha = 0.75;  /* Alpha and beta values */
  double beta  = 0.5;
  double sigma = 1.0;   /* Standard distributions */
  double mu_0 = 0.0;

  /* Relative error tolerances */
  int    Ntol = 5;
  double reltol[] = {1.2e-14, 1e-12, 1e-10, 1e-8, 1e-6};

  /* Execution times */
  double t_total = 0;
  time_t rawtime;
  struct tm * timeinfo;
  char hour[20];
  double t;

#ifdef _WIN32
  long t_1, t_2, t_ini, t_fin;
#else
  struct timeval t_ini, t_fin, t_1, t_2;
#endif

  /* Output file and filename */
  FILE * ftimes;
  char nametimes[256];

  /* STABLE program input file */
  FILE * f_in;

  char marker[]= {'-','\\','|','/'};

  /* x-axis sweep:
   * Avoid too negative values where PDF and CDF approache zero.
   * -10 to   10 step 0.015625, (1280 points)
   *  10 to  100 step 0.12500,  ( 720 points)
   * 100 to 1000 step 1.00000,  ( 901 points) => total of 2901 points */
  /*
  int    Ndiv    = 3;
  int    Nxdiv[] = {     641,   721, 901  };
  double    x0[] = {     -10,    10, 100  };
  double xstep[] = { .015625,  0.25, 1.0  };
  */

  /* x-axis sweep
   * x values have been chosen to cover the 1%-99% inter-percentile range.
   * To achieve this, x0 and xstep must be adapted for each alpha and beta.
   */
  int       Nx = 10000;
  double  x0[] = { -40, 272.46875 };
  double xstep = .03125;

  /* Check input arguments */
  if ( argc == 3 )
    {
      nfun       = atoi ( argv[1] );
      Ntest   = atoi ( argv[2] );
    }
  if ( nfun < 1 || nfun > 2 || argc !=3 )
    {
      printf ( "Usage: nolan_performance FUNC NTESTS\n" );
      printf ( "       FUNC = 1: PDF\n" );
      printf ( "       FUNC = 2: CDF\n" );
      exit ( 2 );
    }

  /* Remove previous outputs of stablec */
  remove ( "stable.out" );

  /* Open output file */
  sprintf ( nametimes, "nolan_performance_%s_%dtest.txt",funname[nfun], Ntest );

  if ( ( ftimes = fopen ( nametimes, "wt" ) ) == NULL )
    {
      printf ( "Error while opening output file." );
      exit ( 3 );
    }

  /* Print header to file */
  fprintf ( ftimes,"NTOL %d\n", Ntol );
  fprintf ( ftimes,"NTESTS %d\n", Ntest );
  fprintf ( ftimes,"FUNCTION %d\n", nfun );
  fprintf ( ftimes,"ALPHA %1.2lf\n", alpha );
  fprintf ( ftimes,"BETA %1.2lf\n", beta );

  printf ( " - John Nolan's STABLE performance benchmark\n" );
  printf ( " - Alpha = %1.2lf, Beta = %1.2lf, Ntest  = %d\n", alpha, beta, Ntest );
  printf ( "\n" );
  printf ( "   TIME      FUNC    RELTOL     0%% -------------------- 100%%\n" );

#ifdef _WIN32
  t_ini = GetTickCount();
#else
  gettimeofday ( &t_ini,NULL );
#endif
  for ( i = 0; i < Ntol; i++ )
    {
      /* Print progress info */
      time ( &rawtime );
      timeinfo = localtime ( &rawtime );
      strftime ( hour,20,"%X",timeinfo );
      printf ( " %s      %d   % 1.2e      ", hour, nfun, reltol[i] );

      /* Print relTol header to file */
      fprintf ( ftimes," RELTOL %1.2e\n",reltol[i] );

      /* Generate an input file for stablec.exe */
      /* Input file format:
      *   5\n        : change stablec settings
      *   %d\n       : change (1) PDF or (2) CDF settings
      *   %1.2e\n    : new reltol value
      *   %d\n       : compute (1) PDF or (2) CDF
      *   %d\n       : parametrization
      *   %f %f %f\n : alpha min, alpha max, alpha step
      *   %f %f %f\n : beta min, beta max, beta step
      *   %f %f %f\n : x min, x max, x step
      *   %f %f\n    : sigma, mu
      *   %d\n       : Output (2) to text file
      */

      f_in = fopen ( "input_for_stablec.dat","wt" );

      fprintf ( f_in,"5\n" );
      fprintf ( f_in,"%d\n",nfun );
      fprintf ( f_in,"%1.2e\n",reltol[i] );
      fprintf ( f_in,"%d\n",nfun );
      fprintf ( f_in,"%d\n",P );
      fprintf ( f_in,"%lf 2 5\n", alpha );
      fprintf ( f_in,"%lf 1 5\n", beta );
      fprintf ( f_in,"%2.9lf %2.9lf %2.9lf\n",	x0[0], x0[1]-.5*xstep, xstep );
      fprintf ( f_in,"%lf %lf\n", sigma, mu_0 );
      fprintf ( f_in,"2\n" );

      fclose ( f_in );
      f_in=NULL;
	  
      /* Repeat execution NTESTS times */
      for ( kt = 0; kt < Ntest; kt++ )
        {
          putchar ( marker[kt%4] );
          fflush ( stdout );

          /* Execute stablec.exe. Output accumulates in stable.out */
#ifdef _WIN32
          t_1 = GetTickCount();
          system ( "stablec.exe < input_for_stablec.dat > sink.txt" );
          t_2 = GetTickCount();
          t = ( double ) ( t_2 - t_1 ) /1000.0;
#else
          gettimeofday ( &t_1,NULL );
          if ( system ( "wine ./stablec.exe < input_for_stablec.dat > sink.txt" ) != 0 )
            {
              printf ( "System call returned error value.\n" );
              exit ( -3 );
            }
          gettimeofday ( &t_2,NULL );
          t=t_2.tv_sec-t_1.tv_sec+ ( t_2.tv_usec-t_1.tv_usec ) /1000000.0;
#endif

          /* Print elapsed time to ftimes file */
          fprintf ( ftimes," %1.9lf ", t );

          /* Points evaluated till now */
          npointsacum += Nx;

          printf ( "\b" );
          /* Print mark for each 5% progress */
          for ( j=0; j < ( kt+1 ) * 20/Ntest - kt*20.0/Ntest; j++ )
	  {
            printf ( "=" );
	  }

        }
        
      fprintf ( ftimes,"\n" ); /* add newline. */
      printf ("\n");
    }

  printf ( "\n" );

#ifdef _WIN32
  t_fin=GetTickCount();
  t_total = ( double ) ( t_fin- t_ini ) /1000.0;
#else
  gettimeofday ( &t_fin, NULL );
  t_total = t_fin.tv_sec-t_ini.tv_sec+ ( t_fin.tv_usec-t_ini.tv_usec ) /1000000.0;
#endif

  printf (" Estimating time offset introduced in free available version of\n");
  printf (" John Nolan's STABLE program.\n");
  printf (" Take time to compute Cauchy distribution as offset, since it can\n");
  printf (" only be calculated via its direct formula.\n");
  printf (" This way, writing to disk overload is also estimated.\n");
  
  alpha = 1.0;
  beta  = 0.0;
  sigma = 1.0;
  mu_0  = 0.0;
 
  /* Generate input file */
  f_in = fopen ( "input_for_stablec.dat","wt" );
  fprintf ( f_in,"%d\n",nfun );
  fprintf ( f_in,"%d\n",P );
  fprintf ( f_in,"%lf 2 5\n", alpha );
  fprintf ( f_in,"%lf 1 5\n", beta );
  fprintf ( f_in,"%1.9lf %1.9lf %1.9lf\n",x0[0],x0[1]-.5*xstep,xstep);
  fprintf ( f_in,"%lf %lf\n", sigma, mu_0 );
  fprintf ( f_in,"2\n" );
  fclose ( f_in );
  f_in=NULL;
  
  /* Repeat execution NTESTS times */
  fprintf(ftimes, "OFFSET_TIME\n");
  for ( kt = 0; kt < Ntest; kt++ )
    {
      putchar ( marker[kt%4] );
      fflush ( stdout );

      /* Execute stablec.exe. Output accumulates in stable.out */
#ifdef _WIN32
      t_1 = GetTickCount();
      system ( "stablec.exe < input_for_stablec.dat > sink.txt" );
      t_2 = GetTickCount();
      t = ( double ) ( t_2 - t_1 ) /1000.0;
#else
      gettimeofday ( &t_1,NULL );
      if ( system ( "wine ./stablec.exe < input_for_stablec.dat > sink.txt" ) != 0 )
	{
	  printf ( "System call returned error value.\n" );
	  exit ( -3 );
	}
      gettimeofday ( &t_2,NULL );
      t=t_2.tv_sec-t_1.tv_sec+ ( t_2.tv_usec-t_1.tv_usec ) /1000000.0;
#endif

      /* Print elapsed time to ftimes file */
      fprintf ( ftimes," %1.9lf ", t );

      /* Print mark for each 5% progress */
      printf ( "\b" );
      for ( i=0; i < ( kt+1 ) * 20/Ntest - kt*20.0/Ntest; i++ ) printf ( "=" );
    }
    
  /* DONE. Add newline and close file */
  fprintf ( ftimes,"\n" ); 
  fclose ( ftimes );
  ftimes=NULL;

  /* Remove garbage */
  remove ( "input_for_stablec.dat" );
  remove ( "stable.out" );
  remove ( "sink.txt" );

  printf ( "\n    %d evaluations in %3.3lf seconds\n", npointsacum,t_total );
  printf ( "    Data in %s\n", nametimes );
  printf ( "    Avg. performance: %d points / %3.3lf s = %lf points/s\n",
	  npointsacum,t_total, ( ( double ) npointsacum ) / ( ( double ) t_total ) );

  return 0;
}

