/** @file nolan_precision.c
 * @brief John Nolan's precision data generator (ground truth).
 * 
 * This benchmark program is employed to obtain the data presented in Table 1
 * and Figure 3 of the manuscript.
 * It estimates the precision of John Nolans's STABLE software when
 * calculating the PDF or CDF of standard alpha-stable distributions.
 * It obtains several calculations of the desired function at:
 *     - A finite set of points in the alpha-beta parameter space.
 *     - A log-scaled sweep on abscissae axis.
 *
 * Usage:
 *     nolan_precision FIG FUN,
 * where:
 *     FIG = 1: Table 1,
 *     FIG = 3: Figure 3,
 *     FUN = 1: PDF data,
 *     FUN = 2: CDF data.
 *           
 * Copyright (C) 2013. Javier Royuela del Val,
 *                     Federico Simmross Wattenberg.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Javier Royuela del Val.
 *  E.T.S.I. Telecomunicacin,
 *  Universidad de Valladolid,
 *  Paseo de Beln 15, 47002 Valladolid, Spain.
 *  jroyval@lpi.tel.uva.es
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef _WIN32
  #include <windows.h>
#else
  #include <sys/time.h>
  #include <time.h>
#endif

#define P 0 /* Use 0 parametrization */
#define THREADS 0 /* Use as many threads as available */

/* #define min(a,b) ((a<b) ? (a) : (b)) */

/***********************
 * Main function
 *********************** */
int
main ( int argc, char *argv[] )
{
  /* generate data for table_1 (n=1) or figure_3 (n=2) */
  int n = 0;
  /* Generate PDF (nfun=1) or CDF (nfun=2) data */
  int nfun =  0;
  char *funname[] = {"", "pdf", "cdf"};
  char *figname[] = {"", "table_1", "", "figure_3"};
  char f_out_name[255];

  /* Points evaluated */
  int npoints;         /* In each iteration */
  int npointsacum = 0; /* Total */

  int Na, Nb;
  double alpha[50];
  double beta[50];
  double sigma = 1, mu_0 = 0;
  double Dbeta;

  /* Output filename */
  // char name[256];

  /* Aux indices */
  int i, j;

  /* Log-scaled x axis division*/
  int Nx = 5417;
  /* 7 segments */
  int    Ndiv = 7;
  /* Segments' edgees */
  double x0[]          = { -1000, -100,  -10,   -1,    1,   10,  100, 1000};
  /* Number of points per segment */
  double Nxdiv[]       = {   900,  720,  576, 1024,  576,  720,  901};
  /* Start position of each segment in the array */
  unsigned int Nx0[]   = {     0,  900, 1620, 2196, 3220, 3796, 4516};
  /* Points separation in each segment. Must be exactly representable */
  double xD[]          = { 1, .125, .015625, .001953125, .015625, .125, 1};
  char ch;


  FILE * f_in;    /* STABLE program input file */
  FILE * f_out;   /* STABLE program output file */

  double t_total;
#ifdef _WIN32
  long t_ini, t_fin;
#else
  struct timeval t_ini, t_fin;
#endif

  /* Check arguments */
  if ( argc == 3 )
    {
      n = atoi ( argv[1] );
      nfun = atoi ( argv[2] );
    }
  if ( argc !=3 || (n != 1 && n != 3) || (nfun != 1 && nfun != 2) )
    {
      printf ( "Usage: nolan_precision FIG FUN\n" );
      printf ( "       FIG = 1: Table 1 data\n" );
      printf ( "       FIG = 3: Figure 3 data\n" );
      printf ( "       FUN = 1: PDF data\n" );
      printf ( "       FUN = 2: CDF data\n" );
      exit ( 2 );
    }

  switch ( n )
    {
    case 1: /* Table 1 data */
      Na = 5;
      Nb = 3;
      Dbeta  = 0.5;
      alpha[0] = 0.25;
      alpha[1] = 0.5;
      alpha[2] = 0.75;
      alpha[3] = 1;
      alpha[4] = 1.5;
      beta[0] = 0;
      beta[1] = 0.5;
      beta[2] = 1;

      break;

    case 3: /* Figure 3 data. Thinner sweep in alpha and beta */
      Na = 40;
      Nb = 17;
      Dbeta = 0.0625;
      for ( i = 0; i < Na; i++ )
        {
          alpha[i] = 0.05* ( i+1 );
        }
      for ( i = 0; i < Nb; i++ )
        {
          beta[i] = 0.0625*i;
        }

      break;

    default:
      printf ( "\nError: no valid n value (1 or 3)\n" );
      return -1;
    }


  /* Remove previous STABLE outputs if present */
  remove ( "stable.out" );

  /* Get start time */
#ifdef _WIN32
  t_ini = GetTickCount();
#else
  gettimeofday ( &t_ini,NULL );
#endif

  /* PDF and CDF data computation */
  /* Calculations are divided in intervals */
  printf ( " Calculating %s data.\n", funname[nfun] );
  printf ( "  ALPHA   INTERVAL\n" );

  for ( i = 0; i < Na; i++ )
    {
      printf ( " % 1.2lf   ", alpha[i] );
      fflush(stdout);

      for ( j = 0; j < Ndiv; j++ )
        {

          /* Generate an input file for stablec.exe for each interval */
          /* Input file format:
          *   %d\n       : 1) PDF, 2) CDF
          *   %d\n       : parametrization
          *   %f %f %f\n : alpha min, alpha max, alpha step
          *   %f %f %f\n : beta min, beta max, beta step
          *   %f %f %f\n : x min, x max, x step
          *   %f %f\n    : sigma, mu
          *   %d\n       : 2) output to text file
          */
          f_in = fopen ( "input_for_stablec.dat","wt" );

          fprintf ( f_in, "%d\n", nfun );
          fprintf ( f_in, "%d\n", P );
          fprintf ( f_in, "%lf 2 5\n", alpha[i] );
          fprintf ( f_in, "%lf %lf %1.9lf\n", beta[0], beta[Nb-1]+.001, Dbeta );
          fprintf ( f_in, "%lf %lf %1.9lf\n", x0[j], x0[j+1]-.5*xD[j], xD[j] );
          fprintf ( f_in, "%lf %lf\n", sigma, mu_0 );
          fprintf ( f_in, "2\n" );

          fclose ( f_in );
          f_in = NULL;

          /* Execute stablec.exe. Output accumulates in stable.out */
#ifdef _WIN32
          system ( "stablec.exe < input_for_stablec.dat > sink.txt" );
#else
	  if ( system ( "wine ./stablec.exe < input_for_stablec.dat > sink.txt" ) != 0)
            {
              printf ( "System call returned error value.\n" );
              exit ( -3 );
            }
#endif
          /* Number of evaluations done (per beta value) */
          npoints = ( int ) ( ( x0[j+1]-0.5*xD[j]-x0[j] ) / xD[j] ) + 1;
          npointsacum += npoints * Nb;

          printf ( "%d (%d p) ", j, npoints );
	  fflush(stdout);
        }
      printf ( "\n" );
    }

  /* Total elapsed time */
#ifdef _WIN32
  t_fin=GetTickCount();
  t_total = ( double ) ( t_fin - t_ini ) /1000.0;
#else
  gettimeofday ( &t_fin, NULL );
  t_total = t_fin.tv_sec - t_ini.tv_sec +
            ( t_fin.tv_usec-t_ini.tv_usec ) /1000000.0;
#endif

  printf ( "\n-----> Done: %d points evaluated in %3.3lf seconds <-----\n",
           npointsacum, t_total );
  printf ( " Avg. performance: %d points / %3.3lf s = %lf points/s\n",
           npointsacum, t_total,
           ( ( double ) npointsacum ) / ( ( double ) t_total ) );


  sprintf ( f_out_name,"nolan_precision_%s_%s.txt",funname[nfun],figname[n] );
  printf ( "\n\n Copying data to %s\n", f_out_name );

 if ( ( f_in = fopen ( "stable.out","rt" ) ) ==NULL )
    {
      printf ( "Error opening \'stable.out\'" );
    }
  rewind ( f_in );

  if ( (  f_out = fopen ( f_out_name,"wt" ) ) ==NULL )
    {
      printf ( "Error creating output file" );
    }
 
  fprintf ( f_out, "FUNCTION=%d ALPHAPOINTS=%d BETAPOINTS=%d XPOINTS=%d\n",
            nfun, Na, Nb, Nx );

  //copiar el archivo
  while ( !feof ( f_in ) )
    {
      ch = getc ( f_in );
      if ( ferror ( f_in ) )
        {
          printf ( "Read Error" );
          clearerr ( f_in );
          break;
        }
      else
        {
          if ( !feof ( f_in ) ) putc ( ch, f_out );
          if ( ferror ( f_out ) )
            {
              printf ( "Write Error" );
              clearerr ( f_out );
              break;
            }
        }
    }

  fclose ( f_in );
  f_in=NULL;
  fclose ( f_out );
  f_out=NULL;

  remove ( "input_for_stablec.dat" );
  remove ( "stable.out" );
  remove ( "sink.txt" );

  return 0;
}

